@@ -49,7 +49,8 @@ C<^[A-Za-z0-9._-]+@[a-zA-Z0-9.-]+\.[a-zA-Z]{2,6}$>
 ######################################################################
 
 sub _check_email {
-  my $value = shift;
+  my ($value) = @_;
+  return '' unless($value);
   # better use rfc822!
   if(! ($value =~ m/^[A-Za-z0-9._-]+@[a-zA-Z0-9.-]+\.[a-zA-Z]{2,6}$/)) {
     return gettext('invalid').'!';
@@ -142,6 +143,16 @@ sub _check_date {
   return '';
 }
 
+######################################################################
+
+=head2 digitonly
+
+... returns I<invalid> if the value doesn't match '[0-9]*'.
+
+=cut
+
+######################################################################
+
 sub _check_digitonly {
   $_ = shift;
   return gettext('invalid').'!' unless(m/^[0-9]*$/);
@@ -150,6 +161,62 @@ sub _check_digitonly {
 
 ######################################################################
 
+=head2 fmatch
+
+... requires the special variable I<fmatch>. This variable must
+contain the name of another field. The value of this field is read in
+and compared with the current value, I<doesn't match> is returned if
+this fails.
+
+When using the same fields several times, you must also define
+I<ROWNUM>, this must start with 1 increased by one whenever the field
+names are repeated.
+
+B<Note:> When you defined several tables, you must reference other
+fields with I<tablename.fieldname>!
+
+=cut
+
+######################################################################
+
+sub _check_fmatch {
+  my($value,$field,$self) = @_;
+  my $match = $self->_get_var('fmatch');
+  if($match) {
+    $_ = $match;
+    my $match = $self->get_input_value($match);
+    print STDERR "no such field: $_" and return '' unless(defined($match));
+    if(ref($match) eq 'ARRAY' and $_ = $self->_get_var('ROWNUM',1)) {
+      $match = $match->[$_-1];
+    }
+    return gettext('doesn\'t match') . '!' if(defined($value) and $value ne $match);
+  }
+}
+
+######################################################################
+
+=head2 regex
+
+... requires the special variable I<regex>, it must contain a valid
+regular expression. If the value doesn't match this regex, I<invalid>
+is returned. 
+
+=cut
+
+######################################################################
+
+sub _check_regex {
+  my($value,$field,$self) = @_;
+  my $regex = $self->_get_var('regex');
+  #print STDERR $regex, " ", $value, "\n";
+  if($regex) {
+    return gettext('invalid').'!' unless(defined($value) and ($value =~ m/$regex/));
+  }
+  return '';
+}
+
+######################################################################
+
 1;
 
 =head1 WRITING A CHECK ROUTINE
@@ -168,6 +235,8 @@ C<$value> contains the submitted field value.
 C<$name> contains the fields name.
 C<$self> contains a reference to the FormEngine object.
 
+B<Note:> you can define the error message by yourself with the variable I<errmsg>!
+
 =head2 Install
 
 If your routine does a general job, you can make it part of FormEngine. Therefore just
