@@ -12,8 +12,6 @@ package HTML::FormEngine::Handler;
 
 use Locale::gettext;
 
-#textdomain('HTML-FormEngine-DBSQL');
-
 ######################################################################
 
 =head2 default
@@ -29,9 +27,7 @@ It then reads in this template and processes it. The resulting code is returned.
 ######################################################################
 
 sub _handle_default {
-  my $self = shift;
-  shift;
-  my $templ = shift;
+  my ($self,$templ) = @_;
   if(defined($templ) && defined($self->{skin}->{$templ})) {
     return $self->_parse($self->{skin}->{$templ});
   }
@@ -71,8 +67,7 @@ if you want to change variable names.
 ######################################################################
 
 sub _handle_checked {
-  my $self = shift;
-  my($res, $keyvar1, $keyvar2, $keyvar3) = split(' ', $_) if $_ = shift;
+  my($self, $caller, $res, $keyvar1, $keyvar2, $keyvar3) = @_;
   $res = 'checked' if(! $res);
   $keyvar1 = 'OPT_VAL' if(! $keyvar1);
   $keyvar2 = 'NAME' if(! $keyvar2);
@@ -94,17 +89,15 @@ sub _handle_checked {
       $self->{_handle_checked}->{$name} = {};
     }
     if($self->{_handle_checked}->{$name}->{$option}) {
-      $self->{i} = 0;
       shift @{$checked};
       foreach $_ (keys(%{$self->{_handle_checked}->{$name}})) {
 	$self->{_handle_checked}->{$name}->{$_} = 0;
       }
     }
-
+    
     $self->{_handle_checked}->{$name}->{$option} = 1;
-
+    
     $input = $checked->[0];
-    $self->{i} ++;
   }
   elsif(ref($checked) eq 'ARRAY') {
     if(grep {$_ eq $value} @{$checked}) {
@@ -115,7 +108,7 @@ sub _handle_checked {
     $input = $checked;
   }
   if($input ne '' && ($input eq $value)) {
-    return " $res";
+    return $res;
   }
   else {
     return '';
@@ -124,12 +117,105 @@ sub _handle_checked {
 
 ######################################################################
 
+=head2 checked_uniq
+
+This handler is designed for checkboxes. With checked_uniq, you only
+have to define one name for all options, but you can't use this name
+again.
+
+The first argument defines the value, which should be returned if a
+certain option was submitted. By default this is 'checked'.
+
+The second argument defines the name of the variable in which the
+option values are stored (default: OPT_VAL).
+
+The third argument defines the name of the variable which defines the
+field name (default: NAME).
+
+=cut
+
+######################################################################
+
+sub _handle_checked_uniq {
+  my($self, $caller, $res, $keyvar1, $keyvar2) = @_;
+  $res = 'checked' if(! $res);
+  $keyvar1 = 'OPT_VAL' if(! $keyvar1);
+  $keyvar2 = 'NAME' if(! $keyvar2);
+  my $checked = $self->_get_value($keyvar2);
+  my $value = $self->_get_var($keyvar1, 1);
+  if(ref($checked) eq 'ARRAY') {
+    return $res if grep {$value eq $_} @{$checked};
+  }
+  else {
+    return $res if($value eq $checked);
+  }
+  return '';
+}
+
+######################################################################
+
+=head2 confirm_checked
+
+This is a confirm handler. It returns the title of an option if the
+option was submitted. Therefore the C<checked> handler is called, with
+the option title as first argument.
+
+The first argument defines the name of the variable in which the
+option values are stored (default: OPT_VAL).  The second argument
+defines the name of the variable which defines the field name
+(default: NAME).
+
+The third argument defines the name of the variable which stores the
+option titles (default: OPTION).
+
+=cut
+
+######################################################################
+
+sub _handle_confirm_checked {
+  my ($self,$caller,$optvalvar,$namevar,$optionvar) = @_;
+  $optvalvar = 'OPT_VAL' unless($optvalvar);
+  $namevar = 'NAME' unless($namevar);
+  $optionvar = 'OPTION' unless($optionvar);
+  my $res = $self->_get_var($optionvar,1) . '<input type="hidden" name="' . $self->_get_var($namevar,1) . '" value="' . $self->_get_var($optvalvar,1) . '">';
+  return _handle_checked($self,$caller,$res,$optvalvar,$namevar,$optionvar);
+}
+
+######################################################################
+
+=head2 confirm_checked_uniq
+
+This handler is very simalar to C<confirm_checked>. The only
+difference is that it calls C<checked_uniq> instead of C<checked>.
+
+=cut
+
+######################################################################
+
+sub _handle_confirm_checked_uniq {
+  my ($self,$caller,$optvalvar,$namevar,$optionvar) = @_;
+  $optvalvar = 'OPT_VAL' unless($optvalvar);
+  $namevar = 'NAME' unless($namevar);
+  $optionvar = 'OPTION' unless($optionvar);
+  my $res = $self->_get_var($optionvar,1) . '<input type="hidden" name="' . $self->_get_var($namevar,1) . '" value="' . $self->_get_var($optvalvar,1) . '">';
+  return _handle_checked_uniq($self,$caller,$res,$optvalvar,$namevar);
+}
+
+######################################################################
+
 =head2 value
 
-This handler returns the value of the field. Therefore the handler
-needs to know the name of the field. The first argument is used
-to tell the handler the name of the variable in which the field name is stored.
-By default this is I<NAME>.
+This handler returns the value of the field.
+
+The first argument defines the value which should be returned if the
+value is empty. By default this is undef.
+
+If the second argument is true (1), the returned value, will be
+returned (for this field name) again next time.
+
+The third argument is used to tell the handler the name of the
+variable in which the field name is stored.  By default this is
+I<NAME>.
 
 If the form wasn't submitted, the fields default value is returned.
 
@@ -138,14 +224,17 @@ If the form wasn't submitted, the fields default value is returned.
 ######################################################################
 
 sub _handle_value {
-  my $self = shift;
-  my $res = $self->_get_value(shift);
+  my ($self,$caller,$none,$same,$namevar) = @_;
+  my $res = $self->_get_value($namevar);
   if(ref($res) eq 'ARRAY') {
-    return shift @{$res};
-  }
-  else {
-    return $res;
+    if($same) {
+      $res = $res->[0];
+    }
+    else {
+      $res =  shift @{$res};
+    }
   }
+  return (defined($res) and $res || $res eq '0') ? $res : $none;
 }
 
 ######################################################################
@@ -167,22 +256,19 @@ else NULL is returned.
 ######################################################################
 
 sub _handle_error {
-  my $self = shift;
-  my ($keyvar, $namevar) = split(' ', shift || 'ERROR NAME');
-  if(! $namevar) {
-    $namevar = 'NAME';
-  }
-  my $templ = shift;
+  my ($self,$caller,$keyvar,$namevar) = @_;
+  $keyvar = 'ERROR' unless($keyvar);
+  $namevar = 'NAME' unless($namevar);
   if($self->is_submitted && $self->{check_error}) {
     my $check = $self->_get_var($keyvar,1);
-    $check = [ $check ] if(ref($check) ne 'ARRAY');
-    if(@{$check}) {
-      my $value = $self->_get_value($namevar,1);
+    $check = [ $check ] if(ref($check) ne 'ARRAY' and $check ne '');
+    if(ref($check) eq 'ARRAY' and @{$check}) {
+      my $value = $self->_get_value($namevar,1,1);
       my $name = $self->_get_var($namevar,1);
       if(ref($value) eq 'ARRAY') {
-	if (ref($self->{_handle_error}) ne 'ARRAY') {
+	if (ref($self->{_handle_error}) ne 'HASH') {
 	  $self->{_handle_error} = {};
-	  push @{$self->{do_before_make}}, sub { my($self) = @_; $self->{_handle_error} = {}; };
+	  push @{$self->{call_before_make}}, sub { my($self) = @_; $self->{_handle_error} = {}; };
 	}
 	$value = $value->[$self->{_handle_error}->{$name}++ || 0];
       }
@@ -194,7 +280,7 @@ sub _handle_error {
 	if(ref($chk) eq 'CODE') {
 	  if($errmsg = &$chk($value, $name, $self)) {
 	    $self->{errcount} ++;
-	    return $errmsg;
+	    return $self->_get_var('errmsg') || $errmsg;
 	  }
 	}
       }
@@ -203,19 +289,45 @@ sub _handle_error {
   return '';
 }
 
+######################################################################
+
+=head2 gettext
+
+The arguments, given to this handler, are passed through gettext and
+then joined together with a spacing blank inbetween. The resulting
+string is returned.
+
+=cut
+
+######################################################################
+
 sub _handle_gettext {
-  my($self, $txt) =  @_;
+  my ($self,$caller) =  (shift,shift);
   my @res;
-  foreach $_ (split(' ', $txt)) {
+  foreach $_ (@_) {
     #if(m/^&[A-Z_]+&$/) {
     #  $_ = $self->_get_var($1);
     #}
-    s/_/ /g;
+    s/\,/,/g;
     push @res, gettext($_);
   }
   return join(' ', @res);
 }
 
+sub _handle_gettext_var {
+  my ($self,$caller) =  (shift,shift);
+  my @res;
+  foreach $_ (@_) {
+    #print STDERR "'", $_, "'\n";
+    $_ = $self->_get_var($_);
+    #print STDERR "'", $_, "'\n";
+    push @res, gettext($_) if($_ or $_ eq '0');
+    #print STDERR "'", $res[@res-1], "'T\n";
+  }
+  return join(' ', @res);
+  #return 'test';
+}
+
 ######################################################################
 
 1;
@@ -228,19 +340,18 @@ __END__
 In general, a handler has the following structure:
 
    sub myhandler {
-     my($self,$args,$callname) = @_;
-     my @args = split(' ', $args);
+     my($self,$callname,@args) = @_;
      # ... some code ... #
      return $res;
    }
 
 C<$self> contains a reference to the FormEngine object.
 
-C<$args> contains the arguments which were passed to the handler (see Skin.pm).
-
 C<$callname> contains the name or synonym which was used to call the handler.
 So it is possible to use the same handler for several, similar jobs.
 
+C<@args> contains the arguments which were passed to the handler (see Skin.pm).
+
 =head2 Install
 
 You have to edit Config.pm to make your handler available.
